"use strict";
var config, logger, preprocessConfig, registration, _evaluate, _grindCoffeeBeans, _preprocessFile;

config = require('./config');

logger = null;

preprocessConfig = null;

registration = function(mimosaConfig, register) {
  var e;
  logger = mimosaConfig.log;
  preprocessConfig = mimosaConfig.coffeescriptPreprocess.values;
  e = mimosaConfig.extensions;
  return register(['add', 'update', 'buildFile'], 'beforeCompile', _grindCoffeeBeans, e.javascript);
};

_evaluate = function(expression) {
  var trimmed;
  trimmed = expression.trim();
  return preprocessConfig[trimmed];
};

_preprocessFile = function(text) {
  var TextProcessor, TokenTypes, lastPosition, newText, pieces, processor, token;
  newText = '';
  lastPosition = 0;
  pieces = [];
  TokenTypes = {
    Text: 0,
    Substitution: 1
  };
  TextProcessor = (function() {
    function TextProcessor(_at_text) {
      this.text = _at_text;
      this.nextTokenSearchStart = 0;
      this.currentToken = null;
      this.startToken = '/%=';
      this.endToken = '%/';
    }

    TextProcessor.prototype.readNextToken = function() {
      var endTokenIndex, searchStart, startTokenIndex, textEnd, textStart;
      if (this.nextTokenSearchStart >= this.text.length) {
        return false;
      }
      if (!this.currentToken || this.currentToken.type === TokenTypes.Substitution) {
        startTokenIndex = this.text.indexOf(this.startToken, this.nextTokenSearchStart);
        textStart = this.nextTokenSearchStart;
        textEnd = startTokenIndex;
        if (startTokenIndex === -1) {
          this.nextTokenSearchStart = this.text.length;
          textEnd = void 0;
        } else {
          this.nextTokenSearchStart = startTokenIndex;
        }
        this.currentToken = {
          text: this.text.substring(textStart, textEnd),
          type: TokenTypes.Text
        };
      } else {
        if (this.text.substring(this.nextTokenSearchStart, this.nextTokenSearchStart + this.startToken.length) !== this.startToken) {
          throw new Error('algorithm error');
        }
        searchStart = this.nextTokenSearchStart + this.startToken.length;
        endTokenIndex = this.text.indexOf(this.endToken, searchStart);
        if (endTokenIndex === -1) {
          throw new Error('unmatched opening token at position: ' + this.nextTokenSearchStart);
        }
        this.nextTokenSearchStart = endTokenIndex + this.endToken.length;
        this.currentToken = {
          text: this.text.substring(searchStart, endTokenIndex),
          type: TokenTypes.Substitution
        };
      }
      return true;
    };

    TextProcessor.prototype.getCurrentToken = function() {
      return this.currentToken;
    };

    return TextProcessor;

  })();
  processor = new TextProcessor(text);
  while (processor.readNextToken()) {
    token = processor.getCurrentToken();
    if (token.type === TokenTypes.Text) {
      newText += token.text;
    } else if (token.type === TokenTypes.Substitution) {
      newText += _evaluate(token.text);
    } else {
      throw new Error('unexpected token type');
    }
  }
  return newText;
};

_grindCoffeeBeans = function(mimosaConfig, options, next) {
  var file, _i, _len, _ref;
  _ref = options.files;
  for (_i = 0, _len = _ref.length; _i < _len; _i++) {
    file = _ref[_i];
    file.inputFileText = _preprocessFile(file.inputFileText);
  }
  return next();
};

module.exports = {
  registration: registration,
  defaults: config.defaults,
  validate: config.validate
};
